#include "Controller.h"

byte Controller::buttons;
byte Controller::unreleased;

void Controller::Init() {
  pinMode(CONTROLLER_BUTTON_PIN_A, OUTPUT);
  pinMode(CONTROLLER_BUTTON_PIN_B, OUTPUT);
  pinMode(CONTROLLER_BUTTON_PIN_C, OUTPUT);
  pinMode(CONTROLLER_BUTTON_PIN_READ, INPUT);
}

void Controller::ReadButtons() {
  buttons = 0;
  for (int j = 0; j < 8; j++) {
    // reset the common in / out pin to low
    pinMode(CONTROLLER_BUTTON_PIN_READ, OUTPUT);
    digitalWrite(CONTROLLER_BUTTON_PIN_READ, LOW);

    // write the 3 bits to the control pins
    digitalWrite(CONTROLLER_BUTTON_PIN_A, j & 1 ? HIGH : LOW);
    digitalWrite(CONTROLLER_BUTTON_PIN_B, j & 2 ? HIGH : LOW);
    digitalWrite(CONTROLLER_BUTTON_PIN_C, j & 4 ? HIGH : LOW);

    // set the common output pin to an input
    pinMode(CONTROLLER_BUTTON_PIN_READ, INPUT);

    // read the common in / out pin
    int set = digitalRead(CONTROLLER_BUTTON_PIN_READ);

    // store the value in the buttons byte
    if (set) {
      byte bit = set << j;
      buttons |= bit;
    } else {
      byte bit = 1 << j;
      unreleased &= 255 - bit;

    }
  }
}

bool Controller::IsPressed(int button) {
  return buttons & button ? true : false;
}

bool Controller::IsPressedAgain(int button) {
  if (unreleased & button) {
    return false;
  }
  return buttons & button ? true : false;
}

int Controller::GetButtons() {
  return buttons;
}

int Controller::GetUnreleased() {
  return unreleased;
}

void Controller::MarkUnreleased(int button) {
  unreleased |= button;
}
