#include "Arduino.h"
#include "DinoController.h"

// needs to be define here with the actual value in the h file
static byte DinoController::dinoHitL[8];
static byte DinoController::dinoHitR[8];

static byte DinoController::dinoTL1[8];
static byte DinoController::dinoTR1[8];
static byte DinoController::dinoBL1[8];
static byte DinoController::dinoBR1[8];
static byte DinoController::dinoBL2[8];
static byte DinoController::dinoBR2[8];

static byte DinoController::JumpCrouch1[8];
static byte DinoController::JumpCrouch2[8];
static byte DinoController::JumpCrouch3[8];

void DinoController::Init() {
  dino_state = DINO_STATE_WALK1_TRANSITION;
  dino_state_time = 0;
  dino_state_duration = 0;
  dino_is_hit = false;
  SetWalk1();
}

void DinoController::Hit() {
  if (dino_is_hit) {
    return;
  }
  dino_state = DINO_STATE_WALK1_TRANSITION;
  dino_state_time = 0;
  dino_state_duration = 0;
  dino_hit_timer = 1.0;
  dino_is_hit = true;
  SetWalk1();
}
void DinoController::Update(float s) {
  dino_state_time += s;
  if (dino_state_time >= dino_state_duration) {
    NextState();
  }
  if (dino_is_hit) {
    dino_hit_timer -= s;
    if (dino_hit_timer <= 0) {
      dino_hit_timer = 0;
      dino_is_hit = false;
    }
  }
}

void DinoController::SetWalk1() {
  int j;
  for (j = 0; j < 8; j++) {
    if (dino_is_hit) {
      dinoTL[j] = dinoHitL[j];
      dinoTR[j] = dinoHitR[j];
    } else {
      dinoTL[j] = dinoTL1[j];
      dinoTR[j] = dinoTR1[j];
    }
    dinoBL[j] = dinoBL1[j];
    dinoBR[j] = dinoBR1[j];
  }
  dino_sprite_change = true;
}

void DinoController::SetWalk2() {
  int j;
  for (j = 0; j < 8; j++) {
    if (dino_is_hit) {
      dinoTL[j] = dinoHitL[j];
      dinoTR[j] = dinoHitR[j];
    } else {
      dinoTL[j] = dinoTL1[j];
      dinoTR[j] = dinoTR1[j];
    }
    dinoBL[j] = dinoBL2[j];
    dinoBR[j] = dinoBR2[j];
  }
  dino_sprite_change = true;
}

void DinoController::SetJumpCrouch() {
  if (dino_is_hit) {
    return;
  }
  int j;
  for (j = 0; j < 8; j++) {
    dinoTL[j] = JumpCrouch1[j];
    dinoTR[j] = JumpCrouch2[j];
    dinoBL[j] = JumpCrouch3[j];
  }
  dino_sprite_change = true;
}

void DinoController::Jump() {
  if (dino_is_hit) {
    return;
  }
  SetJumpCrouch();
  dino_state = DINO_STATE_JUMP;
  dino_state_duration = 1.25;
}

void DinoController::Crouch() {
  if (dino_is_hit) {
    return;
  }
  SetJumpCrouch();
  dino_state = DINO_STATE_CROUCH;
  dino_state_duration = 1.25;
}

void DinoController::NextState() {
  switch (dino_state) {
    case DINO_STATE_WALK1_TRANSITION:
      SetWalk1();
      dino_state = DINO_STATE_WALK1;
      dino_state_duration = 0.250;
      break;
    case DINO_STATE_WALK1:
      dino_state = DINO_STATE_WALK2_TRANSITION;
      dino_state_duration = 0;
      break;
    case DINO_STATE_WALK2_TRANSITION:
      SetWalk2();
      dino_state = DINO_STATE_WALK2;
      dino_state_duration = 0.250;
      break;
    case DINO_STATE_WALK2:
      dino_state = DINO_STATE_WALK1_TRANSITION;
      dino_state_duration = 0;
      break;
    case DINO_STATE_CROUCH:
    case DINO_STATE_JUMP:
      dino_state = DINO_STATE_WALK1_TRANSITION;
      dino_state_duration = 0;
      break;

  }

  dino_state_time = 0;
}
